"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const xstate_1 = require("xstate");
const services = __importStar(require("./services"));
const clientMachine = xstate_1.Machine({
    initial: 'idle',
    states: {
        idle: {
            on: {
                CONNECT: {
                    target: 'connecting',
                },
            },
        },
        connecting: {
            invoke: {
                id: 'connect',
                src: services.connect,
                onDone: {
                    target: 'connected',
                    actions: [services.onConnectSuccess()],
                },
                onError: [
                    {
                        target: 'connecting',
                        cond: context => context.reconnect &&
                            context.backoff.attempts <= context.reconnectAttempts,
                    },
                    { target: 'error' },
                ],
            },
        },
        connected: {
            invoke: {
                id: 'processOperations',
                src: services.processOperations,
            },
            on: {
                // connections has been closed :(
                DISCONNECTED: [
                    { target: 'reconnecting', cond: ctx => ctx.reconnect },
                    { target: 'error' },
                ],
                DISCONNECT: {
                    target: 'disconnecting',
                },
            },
        },
        disconnecting: {
            invoke: {
                id: 'disconnect',
                src: services.disconnect,
                onDone: {
                    target: 'idle',
                    actions: [services.onDisconnectSuccess()],
                },
                onError: {
                    target: 'error',
                },
            },
        },
        reconnecting: {
            invoke: {
                id: 'reconnect',
                src: services.reconnect,
                onDone: {
                    target: 'reconnected',
                    actions: [services.onReconnectSuccess()],
                },
                onError: {
                    target: 'error',
                },
            },
        },
        reconnected: {
            on: {
                // connections has been closed :(
                DISCONNECTED: [
                    { target: 'reconnecting', cond: ctx => ctx.reconnect },
                    { target: 'error' },
                ],
                DISCONNECT: {
                    target: 'disconnecting',
                },
            },
        },
        // on error remove all pending operations and close are open operations
        error: {},
    },
});
exports.clientMachine = clientMachine;
exports.default = clientMachine;
