"use strict";
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = require("aws-sdk");
const iterall_1 = require("iterall");
const formatMessage_1 = __importDefault(require("./formatMessage"));
const execute_1 = __importDefault(require("./execute"));
class PubSub {
    constructor(events) {
        this.events = events;
    }
    asyncIterator(eventNames) {
        const names = Array.isArray(eventNames) ? eventNames : [eventNames];
        return iterall_1.createAsyncIterator(this.events.filter(event => names.includes(event.event)).map(event => event.payload));
    }
}
// polyfill Symbol.asyncIterator
if (Symbol["asyncIterator"] === undefined) {
    Symbol["asyncIterator"] = Symbol.for("asyncIterator");
}
const db = new aws_sdk_1.DynamoDB.DocumentClient();
function createDynamoDBEventProcessor({ connectionManager, schema, subscriptionManager }, eventsTableName = "Events") {
    return async function processDynamoDBStreamEvents({ Records }) {
        var e_1, _a;
        for (const record of Records) {
            // process only INSERT events
            if (record.eventName !== "INSERT") {
                continue;
            }
            // now construct event from dynamodb image
            const event = aws_sdk_1.DynamoDB.Converter.unmarshall(record.dynamodb.NewImage);
            const deleteEvent = async () => {
                const id = record.dynamodb.Keys["id"]["S"];
                const result = await db
                    .batchWrite({
                    RequestItems: {
                        [eventsTableName]: [
                            {
                                DeleteRequest: {
                                    Key: {
                                        id
                                    }
                                }
                            }
                        ]
                    }
                })
                    .promise();
                console.log(JSON.stringify(result.$response));
            };
            try {
                // iterate over subscribers that listen to this event
                // and for each connection:
                //  - create a schema (so we have subscribers registered in PubSub)
                //  - execute operation from event againt schema
                //  - if iterator returns a result, send it to client
                //  - clean up subscriptions and follow with next page of subscriptions
                //  - if the are no more subscriptions, process next event
                // make sure that you won't throw any errors otherwise dynamo will call
                // handler with same events again
                for (var _b = __asyncValues(subscriptionManager.subscribersByEventName(event.event)), _c; _c = await _b.next(), !_c.done;) {
                    const subscribers = _c.value;
                    const promises = subscribers
                        .map(async (subscriber) => {
                        // create PubSub for this subscriber
                        const pubSub = new PubSub([event]);
                        // execute operation by executing it and then publishing the event
                        const iterable = await execute_1.default({
                            connectionManager,
                            subscriptionManager,
                            schema,
                            connection: subscriber.connection,
                            operation: subscriber.operation,
                            pubSub: pubSub,
                            registerSubscriptions: false,
                            useSubscriptions: true
                        });
                        // remove from db
                        if (!iterall_1.isAsyncIterable(iterable)) {
                            // something went wrong, probably there is an error
                            console.log(iterable);
                            await deleteEvent();
                            return;
                        }
                        const iterator = iterall_1.getAsyncIterator(iterable);
                        const result = await iterator.next();
                        if (result.value != null) {
                            return connectionManager.sendToConnection(subscriber.connection, formatMessage_1.default({
                                id: subscriber.operationId,
                                payload: result.value,
                                type: "GQL_OP_RESULT"
                            }));
                        }
                    })
                        .map(promise => promise.catch(e => console.log(e)));
                    await Promise.all(promises);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            await deleteEvent();
        }
    };
}
exports.createDynamoDBEventProcessor = createDynamoDBEventProcessor;
exports.default = createDynamoDBEventProcessor;
