"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const contentType = __importStar(require("content-type"));
const querystring = __importStar(require("querystring"));
const errors_1 = require("./errors");
const execute_1 = __importDefault(require("./execute"));
class HTTPError extends errors_1.ExtendableError {
    constructor(statusCode, message) {
        super(message);
        this.statusCode = statusCode;
    }
}
function parseGraphQLParams(event) {
    switch (event.httpMethod) {
        case 'GET': {
            // get cannot have a body so parse operation from query params
            throw new HTTPError(405, 'Method not allowed');
        }
        case 'POST': {
            const parsedType = contentType.parse(event.headers['Content-Type']);
            switch (parsedType.type) {
                case 'application/json': {
                    return JSON.parse(event.body);
                }
                case 'application/x-www-form-urlencoded': {
                    return querystring.parse(event.body);
                }
                default: {
                    throw new HTTPError(400, 'Invalid request content type');
                }
            }
        }
        default: {
            throw new HTTPError(405, 'Method not allowed');
        }
    }
}
function createHttpHandler({ connectionManager, schema, formatResponse = JSON.stringify, }) {
    return async function serveHttp(event) {
        try {
            const operation = parseGraphQLParams(event);
            const result = await execute_1.default({
                connectionManager,
                operation,
                schema,
                connection: {},
                pubSub: {},
                subscriptionManager: {},
                useSubscriptions: false,
            });
            return {
                body: formatResponse(result),
                headers: {
                    'Content-Type': 'application/json',
                },
                statusCode: 200,
            };
        }
        catch (e) {
            if (e instanceof HTTPError) {
                return { statusCode: e.statusCode, body: e.message };
            }
            return { statusCode: 500, body: e.message || 'Internal server error' };
        }
    };
}
exports.createHttpHandler = createHttpHandler;
exports.default = createHttpHandler;
